@@ -48,11 +48,11 @@ def ValidateFunctionArguments(func: ast_function_definition.FunctionDefinition,
         var = scope.FindVariable(name)
         if var is None:
             if arg.Token.Lexeme == 'params': return
-            raise exceptions.RuntimeError(f'Function argument {arg.Token.Lexeme} not found.', arg.Token.Location, filename)
+            raise exceptions.RuntimeError(f'Function ({func.Name}) argument {arg.Token.Lexeme} not found.', arg.Token.Location, filename)
         if arg.ExprType == types.ExprTypes.FunctionArgument:
             t = var.GetType().lower()
             if arg.Token.Lexeme != 'any' and arg.Token.Lexeme != 'params' and arg.Token.Lexeme != t:
-                raise exceptions.RuntimeError(f'Invalid function argument type {t}, {arg.Token.Lexeme} type was required.', arg.Token.Location, filename)
+                raise exceptions.RuntimeError(f'Invalid function ({func.Name}) argument type {t}, {arg.Token.Lexeme} type was required.', arg.Token.Location, filename)
 
 
         
@@ -80,6 +80,7 @@ class Interpreter:
 
 
     def InterpretInstruction(self, instruction: ast.AST, scope):
+        #print("Interpret Instruction: ", instruction, " token: ", instruction.Token)
         if instruction.Type == types.AstTypes.Assignment:
             return self.InterpretAssign(instruction, scope)
         if instruction.Type == types.AstTypes.Block:
@@ -157,20 +158,23 @@ class Interpreter:
                         val.Value[i.Value] = value
                         return
                     else:
-                        raise exceptions.RuntimeError(f'Index ({i.Value}) not found.', ind.Token.Location, self.filename)
+                        raise exceptions.RuntimeError(f'Index ({i.Value}) not found. ({var.Name}).', ind.Token.Location, self.filename)
             elif val.Type == types.VariableTypes.Array or val.Type == types.VariableTypes.String:
                 if i.Type != types.VariableTypes.Int:
-                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}), Int index type required.', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Invalid index type ({i.Type}), Int index type required. ({var.Name}).', ind.Token.Location, self.filename)
                 if i.Value < 0 or i.Value >= len(val.Value):
-                    raise exceptions.RuntimeError(f'Index ({i.Value}) is out of range.', ind.Token.Location, self.filename)
+                    raise exceptions.RuntimeError(f'Index ({i.Value}) is out of range. ({var.Name})', ind.Token.Location, self.filename)
             val = val.Value[i.Value]
             cnt+=1
-        if expr.Right.ExprType == types.ExprTypes.NewArrayIndex:
+        if expr.Right is not None and expr.Right.ExprType == types.ExprTypes.NewArrayIndex:
             if val.Type != types.VariableTypes.Array:
                 raise exceptions.RuntimeError(f'Type error, append not supported on {val.Type}', ind.Token.Location, self.filename)    
             val.Value.append(value)
         else:
-            val.Value = value
+            if val.Type == types.VariableTypes.Record:
+                val.Value = value
+            else:
+                val.Value = value.Value
 
 
 
@@ -349,9 +353,12 @@ class Interpreter:
                     raise exceptions.RuntimeError(f'Index ({i.Value}) is out of range.', ind.Token.Location, self.filename)
             else:
                 raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}', ind.Token.Location, self.filename)
-            val = val.Value[i.Value]
+            if val.Type == types.VariableTypes.String:
+                val = value_item.ValueItem().SetFromMacal(val.Token, val.Type, val.Value[i.Value])
+            else:
+                val = val.Value[i.Value]
         if expr.Right is not None and expr.Right.ExprType == types.ExprTypes.NewArrayIndex:
-            raise exceptions.RuntimeError(f'Append not supported on right hand side', ind.Token.Location, self.filename)    
+            raise exceptions.RuntimeError(f'Append not supported on right hand side', ind.Token.Location, self.filename)
         return val
 
 
@@ -652,8 +659,22 @@ class Interpreter:
 
 
 
+    def checkDataArray(self, arr, msg):
+        if arr is None: return
+        if not isinstance(arr, value_item.ValueItem):
+            raise exceptions.RuntimeError(f'checkDataArray ({msg}), arr is not a value item and it should be.', None, self.filename)
+        if isinstance(arr.Value, value_item.ValueItem):
+            raise exceptions.RuntimeError(f'checkDataArray ({msg}), arr.Value is a value item and it should not be.', None, self.filename)
+        if arr.Type == types.VariableTypes.Array and len(arr.Value) > 0:
+            for rec in arr.Value:
+                if not isinstance(rec, value_item.ValueItem):
+                    raise exceptions.RuntimeError(f'checkDataArray ({msg}), arr.Value[n] is not a value item and it should be.', None, self.filename)
+
+
+
     def InterpretSelect(self, instruction: ast_select.Select, scope: mscope.Scope) -> None:
-        data = self.InterpretExpression(instruction.From, scope)       
+        data = self.InterpretExpression(instruction.From, scope)
+        self.checkDataArray(data, 'data after init')
         into = self.EvaluateIntoVariable(instruction.Into, data, scope)
         if into is None:
             if instruction.Merge is True:
@@ -664,20 +685,39 @@ class Interpreter:
             else:
                 intoVar.Value = value_item.ValueItem().SetFromMacal(instruction.Token, types.VariableTypes.Array, [])
             scope.AddVariable(intoVar)
-            into = intoVar.Value
+            into = intoVar.Value      
+        self.checkDataArray(into, 'into after init')
         if instruction.Where is not None and data is not None:
             data = self.InterpretSelectWhere(instruction.Where, data, scope)
+        self.checkDataArray(data, 'data after where')
+
         if not(len(instruction.Fields) == 1 and instruction.Fields[0].Name == '*'):
             data = self.ApplyFieldFilters(data, instruction.Fields)
+        self.checkDataArray(data, 'data after apply field filters')
+
         if instruction.Distinct is True and data is not None and data.Type == types.VariableTypes.Array and len(data.Value) > 0:
-            data = data.Value[0]
+            if isinstance(data.Value[0], value_item.ValueItem):
+                data = data.Value[0]
+            else:
+                raise exceptions.RuntimeError(f'Debug Select (distinct): data.Value[0] is NOT ValueItem but it should!', data.Token.Location, self.filename)    
+        self.checkDataArray(data, 'data after distinct')
         if data is not None and data.Type == types.VariableTypes.Array and len(data.Value) == 1:
-            data = value_item.ValueItem().SetFromMacal(data.Value[0].Token, types.VariableTypes.Record, data.Value[0])
+            if isinstance(data.Value[0], value_item.ValueItem):
+                data = data.Value[0]
+            else:
+                raise exceptions.RuntimeError(f'Debug Select (array of one 1): data.Value[0] is NOT ValueItem but it should!', data.Token.Location, self.filename)              
+        self.checkDataArray(data, 'data after rec[0] 1st')
         if instruction.Merge is True and data is not None:
             data = self.InterpretSelectMerge(into, data)
+        self.checkDataArray(data, 'data after merge')
         # second time is needed, just in case merge returns an array of 1 record..
         if data is not None and data.Type == types.VariableTypes.Array and len(data.Value) == 1:
-            data = value_item.ValueItem().SetFromMacal(data.Token, types.VariableTypes.Record, data.Value[0])
+            if isinstance(data.Value[0], value_item.ValueItem):
+                data = data.Value[0]
+            else:
+                data = value_item.ValueItem().SetFromMacal(data.Token, types.VariableTypes.Record, data.Value[0])
+                raise exceptions.RuntimeError(f'Debug Select (array of one 2): data.Value[0] is NOT ValueItem but it should!', data.Token.Location, self.filename)
+        self.checkDataArray(data, 'data after rec[0] 2nd')
         if data is not None:
             self.SetIntoVariableValue(instruction.Into, data, scope)
 
@@ -733,13 +773,22 @@ class Interpreter:
 
 
 
-    def ApplyFieldFilters(self, data: value_item.ValueItem, fields: typing.List[ast_select_field.SelectField]):
+    def GetEmptyRecord(self, fields: typing.List[ast_select_field.SelectField]) -> value_item.ValueItem:
+        return_data = value_item.ValueItem().SetFromMacal(token.LexToken.nullToken(), types.VariableTypes.Record, {})
+        for fld in fields:
+            return_data.Value[fld.AsName] = value_item.ValueItem().SetFromMacal(None, types.VariableTypes.Nil, types.VariableTypes.Nil)
+        return return_data
+
+
+
+    def ApplyFieldFilters(self, data: value_item.ValueItem, fields: typing.List[ast_select_field.SelectField]) -> value_item.ValueItem:
+        # in select we already gate for * as field, so we don't have to check for that.
         if data is None:
-            returnData = value_item.ValueItem().SetFromMacal(rec.Token, rec.Type, {})
-            for fld in fields:
-                returnData.Value[fld.AsName] = value_item.ValueItem().SetFromMacal(None, types.VariableTypes.Nil, types.VariableTypes.Nil)
+            return self.GetEmptyRecord(fields)
         elif data.Type == types.VariableTypes.Array:
-            returnData = value_item.ValueItem().SetFromMacal(data.Token, data.Type, [])
+            if len(data.Value) == 0:
+                return self.GetEmptyRecord(fields)
+            return_data = value_item.ValueItem().SetFromMacal(data.Token, data.Type, [])
             for rec in data.Value:
                 r = value_item.ValueItem().SetFromMacal(rec.Token, rec.Type, {})
                 for fld in fields:
@@ -747,19 +796,17 @@ class Interpreter:
                         r.Value[fld.AsName] = rec.Value[fld.Name]
                     else:
                         r.Value[fld.AsName] = value_item.ValueItem().SetFromMacal(None, types.VariableTypes.Nil, types.VariableTypes.Nil)
-                returnData.Value.append(r)
+                return_data.Value.append(r)
+            return return_data
         elif data.Type == types.VariableTypes.Record:
-            rec = data
-            r = value_item.ValueItem().SetFromMacal(rec.Token, rec.Type, {})
+            r = value_item.ValueItem().SetFromMacal(data.Token, data.Type, {})
             for fld in fields:
-                if fld.Name in rec.Value:
-                    r.Value[fld.AsName] = rec.Value[fld.Name]
+                if fld.Name in data.Value:
+                    r.Value[fld.AsName] = data.Value[fld.Name]
                 else:
                     r.Value[fld.AsName] = value_item.ValueItem().SetFromMacal(None, types.VariableTypes.Nil, types.VariableTypes.Nil)
-            returnData = r
-        else: # just not filter if it's something we didn't anticipate.
-            returnData = data
-        return returnData
+            return r
+        return data
 
 
 
@@ -806,13 +853,13 @@ class Interpreter:
                 returnData.Value.append(rec)
         # Into can be nil here, so we must check.
         elif into.Type == types.VariableTypes.Record:
-            returnData.Value.append(into.Value)
+            returnData.Value.append(into)
         if data.Type == types.VariableTypes.Array:
             for rec in data.Value:
                 returnData.Value.append(rec)
         # Data should always be record or array, but just to be sure we check.
         elif data.Type == types.VariableTypes.Record:
-            returnData.Value.append(data.Value)
+            returnData.Value.append(data)
         return returnData
 
 
@@ -944,7 +991,10 @@ class Interpreter:
                     raise exceptions.RuntimeError(f'Index ({i.Value}) is out of range.', ind.Token.Location, self.filename)
             else:
                 raise exceptions.RuntimeError(f'Type error, index not supported on {val.Type}', ind.Token.Location, self.filename)
-            val = val.Value[i.Value]
+            if val.Type == types.VariableTypes.String:
+                val = value_item.ValueItem().SetFromMacal(val.Token, val.Type, val.Value[i.Value])
+            else:
+                val = val.Value[i.Value]
         if expr.Right is not None and expr.Right.ExprType == types.ExprTypes.NewArrayIndex:
             raise exceptions.RuntimeError(f'Append not supported on right hand side', ind.Token.Location, self.filename)    
         return val
